#include "vigenere.h"

#include <iterator>
#include <algorithm>
#include <functional>
#include <boost/bind.hpp>

using std::string;
using std::vector;
using std::remove_copy_if;

std::string vigenere::crypt(const string &message) const
{
  vector<unsigned short> cmess=code(transform(message));


  vector<unsigned short> key=extend_key(cmess.size());

  const unsigned short vs=26;
  std::transform(
      cmess.begin(),
      cmess.end(),
      key.begin(),
      cmess.begin(),
      boost::bind(std::modulus<unsigned short>(),boost::bind(std::plus<unsigned short>(),_1,_2),vs)      
      );

  return decode(cmess);
}

unsigned short minusmod26(unsigned short i, unsigned short j)
{
  short k=(i-j)%26;

  if ( k<0 )
    k+=26;

  return k;
}

std::string vigenere::decrypt(const string &message) const
{
  vector<unsigned short> cmess=code(transform(message));

  vector<unsigned short> key=extend_key(cmess.size());

  std::transform(
      cmess.begin(),
      cmess.end(),
      key.begin(),
      cmess.begin(),
      minusmod26
      );

  return decode(cmess);
}

bool notLetter(char ch)
{
  return (ch<'A' || ch>'Z') && (ch<'a' || ch>'z');
}

char toUpper(char ch)
{
  if ( ch >='a' && ch <='z')
    return ch-'a'+'A';
  return ch;
}

string vigenere::transform(const string &st)
{
  string ret;
  remove_copy_if(st.begin(), st.end(), back_inserter(ret), notLetter);
  std::transform(ret.begin(),ret.end(),ret.begin(),toUpper);
  return ret;
}

unsigned short toUint(char ch)
{
  return ch-'A';
}

vector<unsigned short> vigenere::code(const string &mess)
{
  vector<unsigned short> ret;
  std::transform(mess.begin(),mess.end(),back_inserter(ret),toUint);
  return ret;
}

char toChar(unsigned int i)
{
  return ( (char) 'A'+i );
}

string vigenere::decode(const vector<unsigned short> &mess)
{
  string ret;
  std::transform(mess.begin(),mess.end(),back_inserter(ret),toChar);
  return ret;
}

vector<unsigned short> vigenere::extend_key(unsigned int n) const
{
  vector<unsigned short> ret;
  ret.reserve(n);

  unsigned int keysz=key_.size();
  for (unsigned short i=0; i<n; ++i)
    ret.push_back(key_[i%keysz]);
  return ret;
}
